@@ -1,7 +1,16 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 """ Arkose: Desktop application container """
 
-import tempfile, os, subprocess, shutil, pwd, time, signal, netaddr, re
+import netaddr
+import os
+import pwd
+import re
+import shutil
+import signal
+import subprocess
+import tempfile
+import time
+
 
 class ArkoseContainer:
     """
@@ -23,21 +32,30 @@ class ArkoseContainer:
     subnet = None
     config = None
 
-    def __init__(self, path = None, fstype = "ext4", fssize = None, network = "none", xserver = "none", dbus = "none", dbusproxy = [], pulseaudio = False, devices = [], ctype = "cow", root = "/", bind = [], restrict = [], cow = []):
+    def __init__(self, path=None, fstype="ext4", fssize=None, network="none",
+                 xserver="none", dbus="none", dbusproxy=[], pulseaudio=False,
+                 devices=[], ctype="cow", root="/", bind=[], restrict=[],
+                 cow=[]):
         """
-            path: Base path used by Arkose to store the containers (default is ~/.arkose)
+            path: Base path used by Arkose to store the containers
+                  (default is ~/.arkose)
             fstype: ext4 or tmpfs
-            fssize: size in MB. Defaults to 2GB for ext4 and 50% of available RAM for tmpfs
+            fssize: size in MB. Defaults to 2GB for ext4
+                    and 50% of available RAM for tmpfs
             network: Network access: "none", "direct" or "filtered"
-            xserver: What kind of X access should be allowed: none, isolated, direct
+            xserver: What kind of X access should be allowed: "none",
+                     "isolated" or "direct"
             dbus: DBUS access: "none", "system", "session" or "both"
-            dbusproxy: Configuration for DBUS proxy, these will be added line by line to the actual config
+            dbusproxy: Configuration for DBUS proxy,
+                       these will be added line by line to the actual config
             pulseaudio: Enable pulseaudio access in the container
             devices: Allow the given devices in the container
-            ctype: Container type, either cow (copy on write) or bind (full bind-mount of the FS)
+            ctype: Container type, either cow (copy on write) or bind
+                   (full bind-mount of the FS)
             root: Root to use for the container (default is /)
             bind: List of paths to bind-mount in the container
-            restrict: List of paths to empty in the container (bind-mounts an empty directory)
+            restrict: List of paths to empty in the container
+                      (bind-mounts an empty directory)
             cow: List of paths to mount as copy-on-write
         """
 
@@ -100,7 +118,6 @@ class ArkoseContainer:
         if type(cow) != list:
             raise Exception("cow must be a list")
 
-
         # Deal with storage path
         if not path:
             if not os.path.exists(os.path.expanduser("~/.arkose/")):
@@ -133,7 +150,9 @@ class ArkoseContainer:
         self.cow = cow
 
         # Get some data on the user
-        self.user = pwd.getpwuid(int(os.getenv("PKEXEC_UID", os.getenv("SUDO_UID", os.getenv("UID", 1000)))))
+        self.user = pwd.getpwuid(int(
+            os.getenv("PKEXEC_UID",
+                      os.getenv("SUDO_UID", os.getenv("UID", os.geteuid())))))
 
         # If anything fails, cleanup and then raise the exception
         try:
@@ -164,24 +183,27 @@ class ArkoseContainer:
 
             # Create the partition
             sparse = open(image, "w+")
-            sparse.truncate(1024*1024*self.fssize)
+            sparse.truncate(1024 * 1024 * self.fssize)
             sparse.close()
 
             # Create the filesystem
             cmd = ['mkfs.ext4', '-O', '^has_journal', '-F', '-q', image]
-            mkfs = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            mkfs = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                    stderr=subprocess.PIPE,
+                                    universal_newlines=True)
             if mkfs.wait() != 0:
                 print(mkfs.stdout.read())
                 print(mkfs.stderr.read())
-                raise Exception("Unable to create ext4 partition at: %s" % image)
+                raise Exception("Unable to create ext4 partition at: %s" %
+                                image)
 
             # Mount it
             self.__mount(image, workdir, "ext4", "loop")
 
-
         elif self.fstype == "tmpfs":
             # Mount it
-            self.__mount("none", workdir, "tmpfs", "size=%s" % str(1024*1024*self.fssize))
+            self.__mount("none", workdir, "tmpfs", "size=%s" %
+                         str(1024 * 1024 * self.fssize))
 
         # Mount a cgroup filesystem
         cgroup = os.path.join(workdir, "cgroup")
@@ -203,9 +225,12 @@ class ArkoseContainer:
             os.mkdir(root_cow)
             try:
                 try:
-                    self.__mount("none", target, "overlayfs", "upperdir=%s,lowerdir=%s" % (root_cow, self.root))
+                    self.__mount("none", target, "overlayfs",
+                                 "upperdir=%s,lowerdir=%s" % (root_cow,
+                                                              self.root))
                 except:
-                    self.__mount("none", target, "aufs", "br=%s:%s" % (root_cow, self.root))
+                    self.__mount("none", target, "aufs",
+                                 "br=%s:%s" % (root_cow, self.root))
             except:
                 raise Exception("No copy-on-write filesystem available")
 
@@ -214,7 +239,7 @@ class ArkoseContainer:
             self.__mount(self.root, target, "none", "bind,ro")
 
         # Mount an empty directory on top of home unless it's already defined
-        if not self.user.pw_dir in self.cow+self.bind+self.restrict:
+        if not self.user.pw_dir in self.cow + self.bind + self.restrict:
             # Make sure the home directory exists
             homedir_target = os.path.join(target, self.user.pw_dir[1:])
             if not os.path.exists(homedir_target):
@@ -228,7 +253,8 @@ class ArkoseContainer:
             if mount[0] == '/':
                 mount = mount[1:]
 
-            new_restrict = os.path.join(workdir, "%s_restrict" % mount.replace("/", "_"))
+            new_restrict = os.path.join(workdir, "%s_restrict" %
+                                                 mount.replace("/", "_"))
             os.mkdir(new_restrict)
 
             new_target = os.path.join(target, mount)
@@ -244,6 +270,10 @@ class ArkoseContainer:
             elif self.ctype == "bind":
                 self.bind.append("/run/resolvconf")
 
+        # When using sssd, allow access to the pipes
+        if os.path.exists("/var/lib/sss/"):
+            self.bind.append("/var/lib/sss/")
+
         # Setup other copy-on-write mounts
         for mount in self.cow:
             # If source doesn't exist, create it
@@ -263,9 +293,11 @@ class ArkoseContainer:
                 os.mkdir(new_target)
             try:
                 try:
-                    self.__mount("none", new_target, "overlayfs", "upperdir=%s,lowerdir=/%s" % (new_cow, mount))
+                    self.__mount("none", new_target, "overlayfs",
+                                 "upperdir=%s,lowerdir=/%s" % (new_cow, mount))
                 except:
-                    self.__mount("none", new_target, "aufs", "br=%s:/%s" % (new_cow, mount))
+                    self.__mount("none", new_target, "aufs",
+                                 "br=%s:/%s" % (new_cow, mount))
             except:
                 raise Exception("No copy-on-write filesystem available")
 
@@ -296,7 +328,8 @@ class ArkoseContainer:
             if mount[0] == '/':
                 mount = mount[1:]
 
-            new_restrict = os.path.join(workdir, "%s_restrict" % mount.replace("/", "_"))
+            new_restrict = os.path.join(workdir, "%s_restrict" %
+                                                 mount.replace("/", "_"))
             os.mkdir(new_restrict)
 
             new_target = os.path.join(target, mount)
@@ -313,15 +346,15 @@ class ArkoseContainer:
             ONLY CALLED FROM __init__()
             Make X work in the container
         """
-        if self.olddisplay:
-            display = self.olddisplay.split(":")
 
         if self.xserver == "none":
             os.unsetenv("DISPLAY")
             return
 
-        # Make sure we have some kind of X server
-        if not display:
+        if self.olddisplay:
+            display = self.olddisplay.split(":")
+        else:
+            # Make sure we have some kind of X server
             raise Exception("No DISPLAY variable in environment")
 
         if self.xserver == "isolated":
@@ -338,7 +371,8 @@ class ArkoseContainer:
 
             # Start the X server
             cmd = ["xpra", "start", ":%s" % xdisplay]
-            subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
 
             # Wait for X to start as xpra returns before it's ready
             time.sleep(2)
@@ -353,9 +387,11 @@ class ArkoseContainer:
 
             # Allow everyone
             cmd = ["xhost", "+"]
-            subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
 
-        # If X server is accessed over the network, don't try to bind- mount the socket
+        # If X server is accessed over the network,
+        # don't try to bind- mount the socket
         if display[0]:
             return
 
@@ -404,33 +440,40 @@ class ArkoseContainer:
             # Get the dbus information from the environment
             current_dbus = os.getenv("DBUS_SESSION_BUS_ADDRESS", None)
             if not current_dbus:
-                raise Exception("No valid DBUS_SESSION_BUS_ADDRESS in environment")
+                raise Exception("No valid DBUS_SESSION_BUS_ADDRESS "
+                                "in environment")
 
             # Create the target if missing
-            dbus_target = os.path.join(self.target, "tmp/%s-dbus_session" % self.name)
+            dbus_target = os.path.join(self.target, "tmp/%s-dbus_session" %
+                                                    self.name)
             self.files.append(dbus_target)
             if os.path.exists(dbus_target):
                 os.remove(dbus_target)
 
             # Generate the config
             proxy_config = os.path.join(self.path, "dbusproxy_config")
-            config = open(proxy_config,"w+")
+            config = open(proxy_config, "w+")
             for rule in self.dbusproxy:
-                if re.match("^.*;.*;.*;.*$",rule):
+                if re.match("^.*;.*;.*;.*$", rule):
                     config.write("%s\n" % rule)
             config.close()
 
             cmd = ["su", str(self.user.pw_name), "-c"]
 
-            # Start the socket proxy between the current socket and the target socket
+            # Start the socket proxy between the current socket
+            # and the target socket
             if os.path.exists("dbus-proxy/arkose-dbus-proxy"):
-                cmd.append("dbus-proxy/arkose-dbus-proxy %s %s" % (dbus_target, proxy_config))
+                cmd.append("dbus-proxy/arkose-dbus-proxy %s %s" %
+                           (dbus_target, proxy_config))
             else:
-                cmd.append("arkose-dbus-proxy %s %s" % (dbus_target, proxy_config))
+                cmd.append("arkose-dbus-proxy %s %s" %
+                           (dbus_target, proxy_config))
             self.dbus_proxy = subprocess.Popen(cmd)
 
             # Generate the new environment variable
-            os.environ["DBUS_SESSION_BUS_ADDRESS"] = "unix:path=/tmp/%s-dbus_session" % self.name
+            os.environ["DBUS_SESSION_BUS_ADDRESS"] = ("unix:path=/tmp/"
+                                                      "%s-dbus_session" %
+                                                      self.name)
 
             # Check if we are currently using a dbus keyring
             keyring_current = os.path.join(self.user.pw_dir, ".dbus-keyrings")
@@ -439,7 +482,8 @@ class ArkoseContainer:
                 return
 
             # Create the target if missing
-            keyring_target = os.path.join(self.target, self.user.pw_dir[1:], ".dbus-keyrings")
+            keyring_target = os.path.join(self.target, self.user.pw_dir[1:],
+                                          ".dbus-keyrings")
             if not os.path.exists(keyring_target):
                 os.mkdir(keyring_target)
 
@@ -454,20 +498,24 @@ class ArkoseContainer:
             Make pulseaudio work in the container
         """
 
-        if self.pulseaudio == False:
+        if not self.pulseaudio:
             return
 
         # If PULSE_SERVER is in the environment, just send it as is
         if os.getenv("PULSE_SERVER", None):
             if self.network == "none":
-                raise Exception("Pulseaudio is enabled with a remote server but no network.")
+                raise Exception("Pulseaudio is enabled with a remote server "
+                                "but no network.")
             return
 
-
         pulse_target = os.path.join(self.target, "tmp/%s-pulse" % self.name)
         self.files.append(pulse_target)
-        cmd = ["su", str(self.user.pw_name), "-c", "pactl load-module module-native-protocol-unix socket=%s auth-cookie-enabled=0" % pulse_target]
-        pactl = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        cmd = ["su", str(self.user.pw_name), "-c",
+               "pactl load-module module-native-protocol-unix socket=%s "
+               "auth-cookie-enabled=0" % pulse_target]
+        pactl = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE,
+                                 universal_newlines=True)
         pactl.wait()
         self.pulse_module = pactl.stdout.read().strip()
 
@@ -515,18 +563,20 @@ class ArkoseContainer:
 
         # Immediately route it to loopback to avoid race conditions
         cmd = ['ip', 'route', 'add', "%s/31" % self.subnet[2], 'dev', 'lo']
-        route = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        route.wait()
+        subprocess.call(cmd, stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE)
 
     def __mount(self, src, dst, fstype, fsoptions=None, verbose=True):
-        """ Wrapper around mount adding mountpoints to the list of FS to unmount """
+        "Wrapper around mount adding mountpoints to the list of FS to unmount."
 
         if fsoptions:
             cmd = ['mount', '-t', fstype, src, dst, "-o", fsoptions]
         else:
             cmd = ['mount', '-t', fstype, src, dst]
 
-        mount = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        mount = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE,
+                                 universal_newlines=True)
         if mount.wait() != 0:
             if verbose:
                 print(mount.stdout.read())
@@ -535,10 +585,9 @@ class ArkoseContainer:
 
         self.mounts.append(dst)
 
-
     def __get_subnet(self):
         """
-            Return the two IPs of a non-routed /31 subnet in 169.254.0.0/16 
+            Return the two IPs of a non-routed /31 subnet in 169.254.0.0/16
             and the subnet itself
         """
 
@@ -560,11 +609,12 @@ class ArkoseContainer:
 
         # Iterate through link-local
         for netrange in range(1, 254):
-            for subnet in netaddr.IPNetwork("169.254.%s.0/24" % netrange).subnet(31):
+            for subnet in netaddr.IPNetwork("169.254.%s.0/24" %
+                                            netrange).subnet(31):
                 net = str(subnet.network)
                 if ip_to_hex(net) not in networks:
-                    dst = str(netaddr.IPAddress(subnet.first+1))
-                    src = str(netaddr.IPAddress(subnet.last-1))
+                    dst = str(netaddr.IPAddress(subnet.first + 1))
+                    src = str(netaddr.IPAddress(subnet.last - 1))
                     return (src, dst, net)
 
         return False
@@ -580,7 +630,7 @@ class ArkoseContainer:
         config = open(self.config, "w+")
         config.write("""## Base config
 lxc.utsname = %s
-lxc.tty = 4
+lxc.tty = 0
 lxc.pts = 1024
 lxc.rootfs = %s
 lxc.console = none
@@ -657,7 +707,9 @@ lxc.network.script.up = %s
             values = line.split(":")
             meminfo_dict[values[0]] = values[1].split()[0].strip()
         meminfo.close()
-        return ((int(meminfo_dict['MemFree']) + int(meminfo_dict['Buffers']) + int(meminfo_dict['Cached'])) / 1024 / 2)
+        return int((int(meminfo_dict['MemFree']) +
+                   int(meminfo_dict['Buffers']) +
+                   int(meminfo_dict['Cached'])) / 1024 / 2)
 
     def run_command(self, command):
         """ Run "cmd" (list) in the container """
@@ -666,7 +718,8 @@ lxc.network.script.up = %s
         signal.signal(signal.SIGTTOU, signal.SIG_IGN)
 
         # Generate terminal initialization script
-        self.terminal_script = os.path.join(self.target, "tmp/%s-terminal" % self.name)
+        self.terminal_script = os.path.join(self.target, "tmp/%s-terminal" %
+                                                         self.name)
         self.files.append(self.terminal_script)
         script = open(self.terminal_script, "w+")
 
@@ -686,15 +739,15 @@ reset -I
         cmd = ["lxc-execute", "-n", self.name, "-f", self.config]
         cmd += ["sh", "/tmp/%s-terminal" % self.name]
 
-        subprocess.Popen(cmd).wait()
+        subprocess.call(cmd)
 
         # Drop the network configuration
         if self.network == "filtered":
-            subprocess.Popen(os.path.join(self.path, "network-down")).wait()
+            subprocess.call(os.path.join(self.path, "network-down"))
 
         # Kill any remaining trace of the container
         cmd = ["lxc-stop", "-n", self.name]
-        subprocess.Popen(cmd).wait()
+        subprocess.call(cmd)
 
     def cleanup(self):
         """ Unmount everything and remove temporary work dir """
@@ -708,9 +761,10 @@ reset -I
 
         if self.pulse_module:
             os.environ.pop("PULSE_SERVER")
-            cmd = ["su", str(self.user.pw_name), "-c", "pactl unload-module %s" % self.pulse_module]
-            pactl = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            pactl.wait()
+            cmd = ["su", str(self.user.pw_name), "-c",
+                   "pactl unload-module %s" % self.pulse_module]
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
 
             # Apparently in some cases race conditions can appear ...
             time.sleep(2)
@@ -727,7 +781,9 @@ reset -I
         # Start unmounting everything, hopefully in the right order
         for path in reversed(self.mounts):
             cmd = ['umount', path]
-            umount = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            umount = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                      stderr=subprocess.PIPE,
+                                      universal_newlines=True)
             if umount.wait() != 0:
                 print(umount.stdout.read())
                 print(umount.stderr.read())
@@ -739,13 +795,17 @@ reset -I
 
         if self.xpra:
             cmd = ["xpra", "stop", ":%s" % self.xpra]
-            subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
 
         if self.network == "filtered":
             # Just in case it's still there, Remove the firewall rules
-            cmd = ["iptables", "-t", "nat", "-D", "POSTROUTING", "-s", self.subnet[1], "-j", "MASQUERADE"]
-            subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
+            cmd = ["iptables", "-t", "nat", "-D", "POSTROUTING", "-s",
+                   self.subnet[1], "-j", "MASQUERADE"]
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
 
             # Remove the temporary route
             cmd = ['ip', 'route', 'del', "%s/31" % self.subnet[2], 'dev', 'lo']
-            subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
+            subprocess.call(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
